/*
	
					The Hacker's Diet

					  Charting Form
					 
		This file contains all the code for generating and
		displaying weight and exercise charts.
		
*/

#include "Hackdiet.h"			

//	Form globals

static char formTitle[MAX_MONTH_YYYY];	// Form title (static because not copied)
static struct hdMonth *m = NULL;		// Unpacked database for month
static int chartCurrentMonth;			// Actual month we're editing
static Boolean noPrev, noNext;			// Database extrema flags for scroll buttons
static RectangleType r;					// Plot area rectangle

#define leftMargin 	30					// Left margin for chart
#define rightMargin 6					// Right margin for chart
#define topMargin 2						// Top margin for chart
#define bottomMargin 24					// Bottom margin for chart
#define AxisOffset	3					// Axis offset from plot area	

#define PlotScaleLine(x1, y1, x2, y2) \
 							r.topLeft.x + leftMargin + (x1),						\
							((r.topLeft.y + r.extent.y) - bottomMargin)	- (y1),		\
							r.topLeft.x + leftMargin + (x2),						\
							((r.topLeft.y + r.extent.y) - bottomMargin)	- (y2)
							
#define PlotLine(x1, y1, x2, y2) WinDrawLine(PlotScaleLine(x1, y1, x2, y2))
							
#define PlotGrayLine(x1, y1, x2, y2) WinDrawGrayLine(PlotScaleLine(x1, y1, x2, y2))
							
#define PlotFilledLine(x1, y1, x2, y2) WinFillLine(PlotScaleLine(x1, y1, x2, y2))
							
//	plotPlanLine  --  Plot a segment of the diet plan on the chart

static void plotPlanLine(UInt16 sx, UInt16 sy, UInt16 ex, UInt16 ey)
{
#define PLAN_PATTERN_LINE

#ifdef PLAN_GRAY_LINE
	PlotGrayLine(sx, sy, ex, ey);
#endif
#ifdef PLAN_PATTERN_LINE
	static CustomPatternType pat[] = { 0xF0, 0xF0, 0xF0, 0xF0,
									   0xF0, 0xF0, 0xF0, 0xF0 };
	
	WinSetPattern(pat);
	PlotFilledLine(sx, sy, ex, ey);
#endif
}

//	drawChart  --  Draw the chart gadget

static void drawChart(FormPtr fp, UInt16 gidget)
{
	int i;
	UInt16 gudget = FrmGetObjectIndex(fp, gidget);
	UInt16 pixelsPerDay, maxLabelRows;
	short lineHeight = FntLineHeight(), charHgt = FntCharHeight();
	static int factors[] = {1, 2, 5};
	int factor = 0, power = 1, vunit = 1;
	weight plotw;
	rung lrung = RUNG_ABSENT;
	weight weightMin, weightMax;		// Weight mininum and maximum
	trend trendMin,trendMax;			// Trend minimum and maximum
	DateTimeType today;
	Boolean plotPlan = false;			// Plot diet plan in chart?
	weight planSwstart, planSwend;		// Plan plot start and end weights
	int planStartDay, planEndDay, planFlatStartDay; // Plan plot day ranges

	TimSecondsToDateTime(TimGetSeconds(), &today);
	FrmGetObjectBounds(fp, gudget, &r);
	WinEraseRectangle(&r, 0);

	pixelsPerDay = (r.extent.x - (leftMargin + rightMargin)) / m->days;
	
	//	If we're using colour, paint the plot area a pretty colour
	
	if (useColour) {
		RectangleType plotRect;
		
		RctSetRectangle(&plotRect, r.topLeft.x + leftMargin - (AxisOffset - 1),
								   r.topLeft.y + topMargin,
								   (pixelsPerDay * m->days) + AxisOffset,
								   (r.extent.y - (topMargin + bottomMargin)) + AxisOffset);
		WinPushDrawState();
		WinSetForeColor(PrefColourIndex(chartBackgroundColour));
		WinSetBackColor(PrefColourIndex(chartBackgroundColour));
		WinFillRectangle(&plotRect, 0);
		WinPopDrawState();
	}
							
	//	Draw axes for chart
		
	PlotLine(-AxisOffset, -AxisOffset, -AxisOffset, r.extent.y - (topMargin + bottomMargin));
	PlotLine(-AxisOffset, -AxisOffset, pixelsPerDay * m->days, -AxisOffset);
		
	//	Label date axis
	
	for (i = 0; i < m->days; i += 7) {
		Char lbl[4];
		Int16 w;
		
		StrPrintF(lbl, "%d", i + 1);
		w = FntCharsWidth(lbl, StrLen(lbl));
		WinDrawChars(lbl, StrLen(lbl),
			r.topLeft.x + leftMargin + (pixelsPerDay * i) - (w / 2),
			((r.topLeft.y + r.extent.y) - bottomMargin) + 4);
	}
	
	//	Plot exercise rung information

#define RungToY(w) ((UInt16) (((((long) (w)) - 1) * (r.extent.y - (bottomMargin + topMargin))) / RUNG_MAX))
	
	if (useColour) {
		WinPushDrawState();
		WinSetForeColor(PrefColourIndex(chartRungColour));
	}
	for (i = 1; i < m->days; i++) {
		if ((m->rungArray[i - 1] != RUNG_ABSENT) && (m->rungArray[i - 1] != 0)) {
			rung rt = m->rungArray[i - 1];
			
			lrung = m->rungArray[i - 1];
			if ((m->rungArray[i] != RUNG_ABSENT) && (m->rungArray[i] != 0)) {
				rt = m->rungArray[i];
			}
			PlotGrayLine(pixelsPerDay * (i - 1), RungToY(m->rungArray[i - 1]),
						 pixelsPerDay * i, RungToY(rt));
		}
	}
	if (useColour) {
		WinPopDrawState();
	}
	
	//	Label rungs at right side of chart (draw only if any rung present)
	
	if (lrung != RUNG_ABSENT) {
		char lrs[12];
		Int16 wid;
		
		StrPrintF(lrs, "%d", lrung);
		wid = FntCharsWidth(lrs, StrLen(lrs));
		WinDrawChars(lrs, StrLen(lrs),
					 (r.topLeft.x + r.extent.x) - wid,
					 ((r.topLeft.y + r.extent.y) - bottomMargin) - (RungToY(lrung) + (charHgt / 2)));
#define RUNG_EXCLUSION_ZONE	4			// How many rungs to exclude around last rung in monthly log
										// (Should really be calculated from font metrics and window
										//	geometry).
		for (i = 1; i <= RUNG_MAX; i = ((i / 6) * 6) + 6) {
			if (abs(lrung - i) >= RUNG_EXCLUSION_ZONE) {
				StrPrintF(lrs, "%d", i);
				wid = FntCharsWidth(lrs, StrLen(lrs));
				WinDrawChars(lrs, StrLen(lrs),
							 (r.topLeft.x + r.extent.x) - wid,
							 ((r.topLeft.y + r.extent.y) - bottomMargin) - (RungToY(i) + (charHgt / 2)));
			}		
		}
	}
				
	//	Find weight and trend extrema for month and establish scale factors
	
	weightMax = 0;
	weightMin = WEIGHT_MAX;
	trendMax = 0;
	trendMin = TREND_MAX;
	for (i = 0; i < m->days; i++) {
		weightMax = max(weightMax, m->weightArray[i]);
		if (m->weightArray[i] != 0) {
			weightMin = min(weightMin, m->weightArray[i]);
		}
		trendMax = max(trendMax, m->trendArray[i]);
		if (m->weightArray[i] != 0) {
			trendMin = min(trendMin, m->trendArray[i]);
		}
	}
	
	/*	If the user wishes to include the diet plan in the chart,
		determine the plan extrema for the month and include them
		in the extrema determined from the weight and trend arrays.  */		
			
	if (PrefFlag(pfPlotPlan) && (prefs.calcStartDate != 0) &&
		(prefs.calcStartWeight != 0) && (prefs.calcGoalWeight != 0)) {
		double startWeight,				// Plan starting weight
			   goalWeight,				// Plan goal weight
			   swstart, swend;			// Sloping weight extrema within month
		DateTimeType dt;
		UInt32 mstart, mend,				// First and last days of month as seconds
			  sstart, send,				// Date range of sloping trend for this month
			  planEndDate;				// End of plan date
		weight pmin = WEIGHT_MAX,		// Plan plot weight extrema
			   pmax = 0;

#define weightToDouble(w)	((w) / 100.0)
#define doubleToWeight(d)	((weight) ((d) * 100))

		//	Calculate the end date of the plan
		
		startWeight = weightToDouble(cvtWeight(prefs.calcStartWeight, prefs.weightUnit, (weightunit) m->weightUnit));
		goalWeight = weightToDouble(cvtWeight(prefs.calcGoalWeight, prefs.weightUnit, (weightunit) m->weightUnit));
		planEndDate = prefs.calcStartDate + ((Int32) ((((goalWeight - startWeight) /
			(((double) prefs.calcCalorieBalance) / CaloriesPerWeightUnit((weightunit) m->weightUnit)))) * 24.0 * 60.0 * 60.0));
	
		//	Get first and last days of month as seconds
	
		dt.year = m->year;
		dt.month = m->month + 1;
		dt.day = 1;
		dt.hour = dt.minute = dt.second = 0;
		mstart = TimDateTimeToSeconds(&dt);
		dt.day = m->days;
		mend = TimDateTimeToSeconds(&dt);
		
		/*	Determine whether the range of dates in the plan contains
			any days of the month we're charting.  If so, then a portion
			of the sloping line from the plan's starting weight to the
			goal will appear on this chart.  Calculate the weight and
			date range of the portion of the plan to be shown on this
			chart.  */
		
		if ((prefs.calcStartDate <= mend) && (planEndDate >= mstart)) {
	
			//	Date range of sloping plan in this month
			
			sstart = max(prefs.calcStartDate, mstart);
			send = min(planEndDate, mend);
			
			//	Weight range of sloping plan in this month
			
			swstart = startWeight + ((goalWeight - startWeight) *
						(((double) (sstart - prefs.calcStartDate)) / (planEndDate - prefs.calcStartDate)));
			swend = startWeight + ((goalWeight - startWeight) *
						(((double) (send - prefs.calcStartDate)) / (planEndDate - prefs.calcStartDate)));
		
			planSwstart = doubleToWeight(swstart);
			planSwend = doubleToWeight(swend);
			pmin = min(pmin, min(planSwstart, planSwend));
			pmax = max(pmax, max(planSwstart, planSwend));
			TimSecondsToDateTime(sstart, &dt);
			planStartDay = dt.day - 1;
			TimSecondsToDateTime(send, &dt);
			planEndDay = dt.day - 1;
		} else {
			planStartDay = -1;				// Plan interval does not overlap this month
		}
		
		/*	Next, we have to consider the case in which the plan ends
			prior to the end of this month (which includes the case where
			the plan ends before the start of the month).  If that
			situation obtains, we need to plot a flat line between the
			end of the plan or the start of the month, whichever is
			later, to the last day of the month.  */
			
		if (planEndDate < mend) {
			sstart = max(planEndDate, mstart);
			TimSecondsToDateTime(sstart, &dt);
			planFlatStartDay = dt.day - 1;
			planSwend = doubleToWeight(goalWeight);
			pmin = min(pmin, planSwend);
			pmax = max(pmax, planSwend);
		} else {
			planFlatStartDay = -1;			// Plan does not end before the end of this month
		}
		
		/*	Roll the extrema of the plan plot into those
			determined from the weight log.  But if there was no
			data in the log, skip plotting the plan.  */

		if (weightMax > 0) {
			weightMax = max(weightMax, pmax);
			weightMin = min(weightMin, pmin);
		}
		
		plotPlan = (planStartDay >= 0) || (planFlatStartDay >= 0);
#undef weightToDouble
#undef doubleToWeight
	}

	weightMax = cvtWeight(weightMax, (weightunit) m->weightUnit, prefs.displayUnit);
	weightMin = cvtWeight(weightMin, (weightunit) m->weightUnit, prefs.displayUnit);
	trendMax = cvtTrend(trendMax, (weightunit) m->weightUnit, prefs.displayUnit);
	trendMin = cvtTrend(trendMin, (weightunit) m->weightUnit, prefs.displayUnit);

	//	Plot weight and trend information, if any present
	
	if (weightMax > 0) {
	
		if (TrendToWeight(trendMax) > weightMax) {
			weightMax = TrendToWeight(trendMax);
		}
		if (TrendToWeight(trendMin) < weightMin) {
			weightMin = TrendToWeight(trendMin);
		}
		
		/*	If there's only one day's worth of data or all the entries
			in the log are identical (it *can* happen), it's possible for
			weightMin and weightMax to be identical, which will result in
			division by zero in the autoscaling and plotting code.  If
			this is the case, we arbitrarily expand the extrema by one
			tenth of a weight unit to plot the data centred on the
			chart.  */
			
		if (weightMin == weightMax) {
			weightMin -= 10;
			weightMax += 10;
		}
		
		maxLabelRows = (r.extent.y - (topMargin + bottomMargin)) / lineHeight;
		
		/*	Find a display scale power and factor which permits
			a suitable number of labels on the weight axis.  We do
			this using units like a vintage Tektronix oscilloscope:
			increasing powers of 1, 2, and 5.  */
			
		factor = 0;
		
		while (((weightMax - (weightMin - (weightMin % vunit))) / (factors[factor] * power)) > maxLabelRows) {
			factor++;
			if (factor > 2) {
				factor = 0;
				power *= 10;
			}
			vunit = factors[factor] * power;
		}
		
		/*	There's no point in using a finer-grained weight unit than
			the one we use to plot weights.  Adjust the weight unit if
			it's smaller than the label increment.  */
			
		if (prefs.displayUnit == wStone) {
			if (vunit < 100) {
				vunit = 100;
			}
		} else {
			if (vunit < 10) {
				vunit = 10;
			}
		}
		
		weightMin -= weightMin % vunit;
		
#define WeightToY(w) ((UInt16) (((((long) (cvtWeight(w, (weightunit) m->weightUnit, prefs.displayUnit))) - weightMin) * (r.extent.y - (bottomMargin + topMargin))) / (weightMax - weightMin)))
#define DisplayWeightToY(w) ((UInt16) (((((long) (w)) - weightMin) * (r.extent.y - (bottomMargin + topMargin))) / (weightMax - weightMin)))
#define TrendToY(t) (WeightToY(TrendToWeight(t)))

		//	Calculate the weekly trend change and calorie balance and add as caption

		if (m->trendArray[0] != 0) {
			signed long weeklyTrendChange, whole, frac;
			char fmt[60], caption[80], deltaw[12], wunits[16], eunits[12],
				 deltacal[12], flagpercent[6];
			char *sgn;
			Int16 wid;
			long dcal;
			double tslope;
			Boolean mslope = false;
			UInt16 nflags = m->days, i, cflags = 0;
			weightunit dwu = prefs.displayUnit;

			if (dwu == wStone) {
				dwu = wPound;
			}
			if ((m->year == today.year) && (m->month == (today.month - 1)) &&
				(m->days > today.day)) {
				UInt16 syear = m->year, smonth = m->month;
				
				nflags = today.day;
				if (smonth == 0) {
					smonth = 11;
					syear--;
				} else {
					smonth--;
				}
				mslope = fitSlopeToMultiMonthTrend(syear, smonth, today.day - 1,
												   m->year, m->month, today.day - 1,
												   &tslope);
			}
			if (!mslope) {
				tslope = fitSlopeToTrend(m, 0, m->days - 1);
			}
			dcal = (long) (EnergyPerWeightUnit(prefs.weightUnit) * tslope);
			weeklyTrendChange = (signed long) (tslope * 1000000 * 7);
			weeklyTrendChange = cvtLong(weeklyTrendChange, prefs.weightUnit, prefs.displayUnit);

			sgn = (weeklyTrendChange < 0) ? "-" : "+";
			if (weeklyTrendChange < 0) {
				weeklyTrendChange = -weeklyTrendChange;
			}
			whole = weeklyTrendChange  / 1000000L;
			frac = ((weeklyTrendChange + 5000L) / 10000L) % 100;
			if (whole == 0 && frac == 0) {
				sgn = "";
			}
			for (i = 0; i < nflags; i++) {
				cflags += (m->flagsArray[i] != 0) ? 1 : 0;
			}
			editDeltaWeight(deltaw, sgn, whole, frac);
			NumOut(deltaw);
			stringListItem(WeightUnitAbbreviationsStringList, dwu, wunits, sizeof wunits);
			StrPrintF(deltacal, "%s%ld", (dcal == 0) ? "" : (dcal > 0 ? "+" : "-"), abs(dcal));					 
			stringListItem(EnergyUnitAbbreviationsStringList, PrefFlag(pfEnergyJoule) ? 1 : 0,
							eunits, sizeof eunits);
			if (cflags != 0) {
				StrPrintF(flagpercent, " %d%%", (cflags * 100) / nflags);
			} else {
				flagpercent[0] = 0;
			}
			stringResource(ChartBalanceFormatString, fmt, sizeof fmt);
			StrPrintF(caption, fmt, deltaw, wunits, deltacal, eunits, flagpercent);
		
			wid = FntCharsWidth(caption, StrLen(caption));
			WinDrawChars(caption, StrLen(caption), r.topLeft.x + (r.extent.x - wid) / 2,
							(r.topLeft.y + r.extent.y) - lineHeight);
		}
		
		/*	If a diet plan exists and the user wishes it plotted
			in the chart, do so now.  */
			
		if (plotPlan) {
			UInt16 sx, sy, ex, ey;
			
			if (useColour) {
				WinPushDrawState();
				WinSetForeColor(PrefColourIndex(chartPlanColour));
				WinSetBackColor(PrefColourIndex(chartBackgroundColour));
			}
			if (planStartDay >= 0) {
				sx = pixelsPerDay * planStartDay;
				sy = WeightToY(planSwstart);
				ex = pixelsPerDay * planEndDay;
				ey = WeightToY(planSwend);
				plotPlanLine(sx, sy, ex, ey);
			}
			if (planFlatStartDay >= 0) {
				sx = pixelsPerDay * planFlatStartDay;
				sy = ey = WeightToY(planSwend);
				ex = pixelsPerDay * m->days;
				plotPlanLine(sx, sy, ex, ey);
			}
			if (useColour) {
				WinPopDrawState();
			}
		}

		//	Plot the trend as a continuous line

		if (useColour) {
			WinPushDrawState();
			WinSetForeColor(PrefColourIndex(chartTrendColour));
		}
		for (i = 0; i < (m->days - 1); i++) {
			if ((m->weightArray[i] != 0) ||
				(m->year < today.year) ||
				((m->year == today.year) && (m->month < (today.month - 1)) ||
				((m->year == today.year) && (m->month == (today.month - 1) && (i <= (today.day - 1)))))) {
				UInt16 oy, ny;
				
				oy = TrendToY(m->trendArray[i]);
				ny = TrendToY(m->trendArray[i + 1]);
				
				PlotLine(pixelsPerDay * i, oy, pixelsPerDay * (i + 1), ny);
			}
		}
		if (useColour) {
			WinPopDrawState();
		}
		
		//	Plot the weights as floats and sinkers
		
		for (i = 0; i < m->days; i++) {
			if (m->weightArray[i] != 0) {
				UInt16 ty, wy;
				int offset;
				
				ty = TrendToY(m->trendArray[i]);
				wy = WeightToY(m->weightArray[i]);
				offset = ((int) wy) - ((int) ty);
				
				if (offset < -2 || offset > 2) {
					if (useColour) {
						WinPushDrawState();
						WinSetForeColor(PrefColourIndex(chartFlsinkColour));
					}
					PlotLine(pixelsPerDay * i, ty + ((offset > 0) ? 1 : -1),
							 pixelsPerDay * i, wy + ((offset > 0) ? -2 : 2));
					if (useColour) {
						WinPopDrawState();
					}
				}
				
				//	If today is Flagged, plot the float or sinker as a solid diamond
				
				if (m->flagsArray[i]) {
					if (useColour) {
						WinPushDrawState();
						WinSetForeColor(PrefColourIndex(chartFlagColour));
					}
					PlotLine(pixelsPerDay * i - 2, wy, pixelsPerDay * i + 2, wy);
					PlotLine(pixelsPerDay * i, wy - 2, pixelsPerDay * i, wy + 2);
					if (useColour) {
						WinPopDrawState();
					}
				}
				
				//	Draw the outline of the float or sinker
				
				if (useColour) {
					WinPushDrawState();
					WinSetForeColor(PrefColourIndex(chartBobColour));
				}
				PlotLine(pixelsPerDay * i - 2, wy, pixelsPerDay * i, wy - 2);
				PlotLine(pixelsPerDay * i, wy - 2, pixelsPerDay * i + 2, wy);
				PlotLine(pixelsPerDay * i + 2, wy, pixelsPerDay * i, wy + 2);
				PlotLine(pixelsPerDay * i, wy + 2, pixelsPerDay * i - 2, wy);
				if (useColour) {
					WinPopDrawState();
				}
			}
		}
		
		//	Label weight axis

#ifdef DUMPSCALE		
{
	char s[80];
	
	StrPrintF(s, "Min = %d Max = %d, Vunit = %d, Rows = %d",
		weightMin, weightMax, vunit, maxLabelRows);
	WinDrawChars(s, StrLen(s), r.topLeft.x, r.topLeft.y);
}
#endif
		for (plotw = weightMin; plotw <= weightMax; plotw += vunit) {
			Int16 wid;
			char txt[12];
			
			editWeight(txt, plotw);
			wid = FntCharsWidth(txt, StrLen(txt));
			WinDrawChars(txt, StrLen(txt),
				r.topLeft.x + (leftMargin - 4) - wid,
				((r.topLeft.y + r.extent.y) - bottomMargin) - (DisplayWeightToY(plotw) + (charHgt / 2)));
		}
	} else {
		//	No weight data in this monthly log!
		
		char s[132];
		Int16 wid;
		
		stringResource(NoDataInLogString, s, sizeof s);
		wid = FntCharsWidth(s, StrLen(s));
		WinDrawChars(s, StrLen(s), r.topLeft.x + (r.extent.x - wid) / 2,
						r.topLeft.y + (r.extent.y / 2));
	}
}

//	tapChart  --  Handle tap in chart

static void tapChart(FormPtr f, UInt16 gindex, RectanglePtr r, int x, int y)
{
	UInt16 day,
		 pixelsPerDay = (r->extent.x - (leftMargin + rightMargin)) / m->days;
	
	if (((x - r->topLeft.x) >= leftMargin) &&
		((x - r->topLeft.x) <= (r->extent.x - rightMargin))) {
		day = ((x - r->topLeft.x) - leftMargin) / pixelsPerDay;
		showSpecificDay = day + 1;
		logGotoMonth = chartCurrentMonth;
		FrmGotoForm(MonthlyLogForm);
	}
}

/*	chartingHandleEvent  --  Handle event while the charting form
							 is displayed.  */
							  

static Boolean chartingHandleEvent(EventPtr event)
{
	Boolean handled;
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif
	handled = false;


	switch (event->eType) {
	  	
  		case frmOpenEvent:
  			//	Note fall-through
		case frmUpdateEvent:
			FrmDrawForm(FrmGetActiveForm());
			drawChart(FrmGetActiveForm(), MonthlyChartChartGadget);
			handled = true;
			break;
			
		case frmCloseEvent:
			if (m != NULL) {
				hdDisposeMonthUnpacked(m);
				m = NULL;
			}
			MenuEraseStatus(NULL);
			FrmEraseForm(FrmGetActiveForm());
			FrmDeleteForm(FrmGetActiveForm());
			handled = true;
			break;
			
		case ctlRepeatEvent:
			switch(event->data.ctlRepeat.controlID) {
				case MonthlyChartPreviousMonthRepeating:	// Previous month
					logGotoMonth = chartCurrentMonth - 1;
changeMonth:		FrmGotoForm(MonthlyChartForm);
					handled = true;
					break;
					
				case MonthlyChartNextMonthRepeating:	// Next month
					logGotoMonth = chartCurrentMonth + 1;
					goto changeMonth;
			}
			break;
			
		case ctlSelectEvent:
			switch(event->data.ctlSelect.controlID) {
				case MonthlyChartDoneButton:			// Done button: Back to month list
					if (m == NULL) {
						monthLogShowToday();
					} else {
						yearGotoYear = m->year;
						FrmGotoForm(YearViewForm);
					}
					handled = true;
					break;
										
				case MonthlyChartTodayButton:			// Today button: Show log entry for today
					{
						DateTimeType dt;
	
						TimSecondsToDateTime(TimGetSeconds(), &dt);
						goToToday = true;
						hdEnsureMonthPresent(dt.year, dt.month - 1);
						logGotoMonth = hdFindMonth(dt.year, dt.month - 1);
						FrmGotoForm(MonthlyLogForm);
						handled = true;
					}
					break;
					
				case MonthlyChartLogButton:			// Log button: Show log for month
viewLog:;			logGotoMonth = chartCurrentMonth;
					FrmGotoForm(MonthlyLogForm);
					handled = true;
					break;
					
				case MonthlyChartTrendButton:		// Trend button: Show trend analysis
					FrmGotoForm(TrendAnalysisForm);
					handled = true;
					break;
			}
			break;
			
		case penDownEvent:
			{
				FormPtr f = FrmGetActiveForm();
				UInt16 g = FrmGetObjectIndex(f, MonthlyChartChartGadget);
				RectangleType r;
				
				FrmGetObjectBounds(f, g, &r);
				if (RctPtInRectangle(event->screenX, event->screenY, &r)) {
					tapChart(f, g, &r, event->screenX, event->screenY);
					handled = true;
				}
			}
			break;
		
		case keyDownEvent:		
			switch (event->data.keyDown.chr) {
					
				case pageUpChr:
					if (!noPrev) {
						logGotoMonth = chartCurrentMonth - 1;
						goto changeMonth;
					}
					handled = true;				// Or, more precisely, ignored
					break;
					
				case pageDownChr:
					if (!noNext) {
						logGotoMonth = chartCurrentMonth + 1;
						goto changeMonth;
					}
					handled = true;
					break;
			}
			break;

		case menuEvent:		
			switch (event->data.menu.itemID) {
				case EditMonthChartPreferences:
					FrmGotoForm(PreferencesForm);
					handled = true;
					break;
						
				case HistoryMonthChartLast30Days:
					historyMonths = 1;
histChart:;			{
						DateTimeType dt;
						
						dt.year = m->year;
						dt.month = m->month + 1;
						dt.day = m->days;
						dt.hour = dt.minute = dt.second = 0;
						historyGoTo = TimDateTimeToSeconds(&dt);
					}
					FrmGotoForm(HistoricalChartForm);
					handled = true;
					break;
					
				case HistoryMonthChartQuarter:
					historyMonths = 3;
					goto histChart;
					
				case HistoryMonthChartSixMonths:
					historyMonths = 6;
					goto histChart;
					
				case HistoryMonthChartYear:
					historyMonths = 12;
					goto histChart;

				case HistoryMonthChartEntireHistory:
					historyMonths = 0;
					goto histChart;
					
				case ViewMonthChartLog:
					goto viewLog;
					
				case ViewMonthChartTrendAnalysis:
					FrmGotoForm(TrendAnalysisForm);
					handled = true;
					break;
					
				case ViewMonthChartThisYear:
					yearGotoYear = m->year;
					FrmGotoForm(YearViewForm);
					handled = true;
					break;						
				
				case ViewMonthChartAllYears:
					FrmGotoForm(YearListForm);
					handled = true;
					break;
					
				case ViewMonthChartAllMonths:
					FrmGotoForm(MonthsForm);
					handled = true;
					break;
					
				case ViewMonthChartExerciseLadder:
					FrmGotoForm(ExerciseForm);
					handled = true;
					break;
					
				case ViewMonthChartDietCalculator:
					FrmGotoForm(DietCalcForm);
					handled = true;
					break;
					
				case ViewMonthChartKilograms:
					prefs.displayUnit = wKilogram;
reChart:;			logGotoMonth = chartCurrentMonth;
					goto changeMonth;
					
				case ViewMonthChartPounds:
					prefs.displayUnit = wPound;
					goto reChart;
					
				case ViewMonthChartStones:
					prefs.displayUnit = wStone;
					goto reChart;
			
				case ViewMonthChartAboutEatWatch:
					aboutDialogue();
					break;
			}
			handled = true;
			break;
	}
#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return handled;
}

/*	chartingLoad  --  Initialise chart form. */

void chartingLoad(FormPtr f)
{
	char monthName[MAX_MONTH_NAME];

	chartCurrentMonth = logGotoMonth;
	m = hdGetMonthUnpacked(chartCurrentMonth);
	
	//	Hide next and/or previous repeating buttons if we're at the end of the database
	
	noNext = noPrev = false;
	if (chartCurrentMonth == 0) {
		CtlHideControl(FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthlyChartPreviousMonthRepeating)));
		noPrev = true;
	}
	if (chartCurrentMonth == (hdNumMonths() - 1)) {
		CtlHideControl(FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthlyChartNextMonthRepeating)));
		noNext = true;
	}
	
	if (m != NULL) {
		FrmSetEventHandler(f, chartingHandleEvent);
	
		stringListItem(MonthNamesStringList, m->month, monthName, sizeof monthName);
		StrPrintF(formTitle, "%s %d", monthName, m->year);
		FrmSetTitle(f, formTitle);
	}
}