/*
	
					The Hacker's Diet

					 Month List Form
					 
		This file contains all the code for displaying and
		interacting with the Month List form, which displays
		a list of monthly log databases currently present on
		the handheld and allows the user to tap on one to
		display it in detail.				
*/

#include "Hackdiet.h"			

//	Form globals

static char **monthListChoices = NULL;
static char *monthListText = NULL;

/*	monthListHandleEvent  --  Handle event while the month list form
							  is displayed.  */
							  
static Boolean monthListHandleEvent(EventPtr event)
{
	Boolean handled;
#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif
	handled = false;


	switch (event->eType) {
			
		case ctlSelectEvent:
			switch(event->data.ctlSelect.controlID) {
				case MonthsTodayButton:			// Today button: Show log for today
					{
						DateTimeType dt;
	
						TimSecondsToDateTime(TimGetSeconds(), &dt);
						goToToday = true;
						hdEnsureMonthPresent(dt.year, dt.month - 1);
						logGotoMonth = hdFindMonth(dt.year, dt.month - 1);
						FrmGotoForm(MonthlyLogForm);
						handled = true;
					}
					break;
					
				case MonthsTrendButton:			// Trend button: Show trend analysis
					FrmGotoForm(TrendAnalysisForm);
					handled = true;
					break;
					
				case MonthsYearsButton:			// Years button: Show year list
					FrmGotoForm(YearListForm);
					handled = true;
					break;
			}
			break;
  	
  		case frmOpenEvent:	
			FrmDrawForm(FrmGetActiveForm());
			handled = true;
			break;
			
		case frmCloseEvent:
			if (monthListChoices != NULL) {
				MemPtrFree(monthListChoices);
				monthListChoices = NULL;
			}
			if (monthListText != NULL) {
				MemPtrFree(monthListText);
				monthListText = NULL;
			}
			MenuEraseStatus(NULL);
			FrmEraseForm(FrmGetActiveForm());
			FrmDeleteForm(FrmGetActiveForm());
			handled = true;
			break;

		case lstEnterEvent:
			LstEraseList(event->data.lstEnter.pList);
			LstSetSelection(event->data.lstEnter.pList, -1);
			LstDrawList(event->data.lstEnter.pList);
			/*	No, you *don't* want to set handled true here.  If you do,
				the list object's own tracking won't occur and scrolling,
				among other things, won't work.
			handled = true;
			*/
			break;
			
		case lstSelectEvent:			//	Since there's only one list, no need to check ID
			{
				FormPtr f = FrmGetActiveForm();
				ListPtr lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthsMonthList));
				
				logGotoMonth = event->data.lstSelect.selection;
				FrmGotoForm(MonthlyLogForm);
				handled = true;
			}
			break;
			
		case keyDownEvent:
		
			/* You'd think that the list object, which dynamically
			   displays a scroll bar when more items are present than
			   fit on the screen, would also respond to the hardware
			   scroll buttons.  But it doesn't.  So the following code
			   handles key down events from the scroll buttons and
			   converts them into LstScrollList() calls to scroll the
			   list accordingly.  The logic is such that the scroll
			   buttons will be no-ops (will never call LstScrollList)
			   unless the list can actually be scrolled in that
			   direction. */
		
			switch (event->data.keyDown.chr) {
				case pageUpChr:
					{
						FormPtr f = FrmGetActiveForm();
						ListPtr lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthsMonthList));
						int lheight = LstGetVisibleItems(lp);
						int newtop = ((int) lp->topItem) - lheight;
						
						if (newtop < 0) {
							newtop = 0;
						}
						if (newtop != lp->topItem) {
							LstScrollList(lp, winUp, lp->topItem - newtop);
						}
						handled = true;
					}
					break;
					
				case pageDownChr:
					{
						FormPtr f = FrmGetActiveForm();
						ListPtr lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthsMonthList));
						int lheight = LstGetVisibleItems(lp);
						int newtop = ((int) lp->topItem) + lheight;
						
						if ((newtop + lheight) > lp->numItems) {
							newtop = lp->numItems - lheight;
						}
						if (newtop < 0) {
							newtop = 0;
						}
						if (newtop != lp->topItem) {
							LstScrollList(lp, winDown, newtop - lp->topItem);
						}
						handled = true;
					}
					break;
			}
			break;

			
		case menuEvent:		
			switch (event->data.menu.itemID) {
#ifndef MonthsOptionsPreferences
#define MonthsOptionsPreferences MonthsPreferences
#define MonthsOptionsTrendAnalysis MonthsTrendAnalysis
#define MonthsOptionsAboutEatWatch MonthsAboutEatWatch
#define MonthsOptionsCSVDebug MonthsCSVDebug
#endif
			
				case MonthsOptionsPreferences:
					FrmGotoForm(PreferencesForm);
					break;
				
				case MonthsOptionsAboutEatWatch:
					aboutDialogue();
					break;
					
				case MonthsOptionsTrendAnalysis:
					FrmGotoForm(TrendAnalysisForm);
					break;
				
					
#ifdef CSV
#ifdef CSVDEBUG
				case MonthsOptionsCSVDebug:
					FrmGotoForm(CSVDebugForm);
					break;
#endif
#endif

						
				case SpecialRecomputeTrend:
					{
						struct hdMonth *mf = hdGetMonthUnpacked(0);
						UInt16 yr, mo;
						const char *otitle;
						char rctitle[80];
						
						stringResource(RecomputingTrendString, rctitle, sizeof rctitle);
						otitle = FrmGetTitle(FrmGetActiveForm());
						FrmEraseForm(FrmGetActiveForm());
						FrmSetTitle(FrmGetActiveForm(), rctitle);
						FrmDrawForm(FrmGetActiveForm());
						mf->trendCarryForward = 0;
						hdPutMonthUnpacked(mf, 0);
						yr = mf->year;
						mo = mf->month;
						hdDisposeMonthUnpacked(mf);
						hdPropagateTrend(yr, mo);
						FrmEraseForm(FrmGetActiveForm());
						FrmSetTitle(FrmGetActiveForm(), (char *) otitle);
						FrmDrawForm(FrmGetActiveForm());
					}
					break;					
			}
			handled = true;
			break;
	}
#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return handled;
}

#ifdef SCRAMBLE_DB
static unsigned long int rand_next = 1;

/* Return next random integer */

static int rand(void)
{
    rand_next = rand_next * 1103515245L + 12345;
    
    return (int) (rand_next / 65536L) % 32768L;
}

/* Set seed for random generator */

static void srand(unsigned int seed)
{
    rand_next = seed;
}
#endif			

/*	monthListLoad  --  Initialise month list form and
					   display.  Called when ApplicationHandleEvent
					   received a frmLoadEvent for this month list
					   form. */

void monthListLoad(FormPtr f)
{
	int iter = -1, n = 0;
	UInt16 nmonths, year, month, thisMonth = 0, lyear = 0;
	ListPtr lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthsMonthList));
	char *cp;
	char monthName[MAX_MONTH_NAME];
#define ListItemMaxTextLength	MAX_MONTH_YYYY
	DateTimeType dt;
		
	FrmSetEventHandler(f, monthListHandleEvent);
	
	TimSecondsToDateTime(TimGetSeconds(), &dt);
	nmonths = hdNumMonths();
	if (nmonths > 0) {
		monthListChoices = MemPtrNew(sizeof(char *) * nmonths);
		if (monthListChoices == NULL) {
			return;
		}
		cp = monthListText = MemPtrNew((ListItemMaxTextLength + 1) * nmonths);
		if (monthListText == NULL) {
			MemPtrFree(monthListChoices);
			monthListChoices = NULL;
			return;
		}
		
		//	Allocation may have moved form and list--recover pointers
		f = FrmGetActiveForm();
		lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthsMonthList));
		
		while (true) {
			hdIterateMonths(&iter, &year, &month);
			if (year == 0 || iter == -1) {
				break;
			}
			assert(n < nmonths);
			if ((year == dt.year) && (month == (dt.month - 1))) {
				thisMonth = n;
			}
			stringListItem(MonthNamesStringList, month, monthName, sizeof monthName);
			StrPrintF(cp, "%s%s %d", (lyear == year) ? "      " : "", monthName, year);
			lyear = year;
			monthListChoices[n] = cp;
			cp += StrLen(cp) + 1;
			n++;
		}
	}
	LstSetListChoices(lp, monthListChoices, n);

#ifdef FORCEUNITS
	{
		int i;
		int Warning_FORCEUNITS_enabled;
		
		for (i = 0; i < n; i++) {
			struct hdMonth *m = hdGetMonthUnpacked(i);
			
			if (m->weightUnit != prefs.weightUnit) {
				m->weightUnit = prefs.weightUnit;
				hdPutMonthUnpacked(m, i);
			}
			hdDisposeMonthUnpacked(m);
		}

		//	Allocation may have moved form and list--recover pointers
		f = FrmGetActiveForm();
		lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthsMonthList));
	}
#endif

#ifdef ROLLING_STONES

	/*	Swoop through the database and convert everything to stones
		as the native log unit.  This allows testing stones with
		databases created using other units.  */
		
	{
		int i;
		int Warning_ROLLING_STONES_enabled;
		
		prefs.calcGoalWeight = cvtWeight(prefs.calcGoalWeight, prefs.weightUnit, wStone);
		prefs.weightUnit = prefs.displayUnit = wStone;
		for (i = 0; i < n; i++) {
			struct hdMonth *m = hdGetMonthUnpacked(i);
			
			if (m->weightUnit != prefs.weightUnit) {
				weightunit ow = (weightunit) m->weightUnit;
				int d;
				
				m->weightUnit = prefs.weightUnit;
				m->trendCarryForward = cvtTrend(m->trendCarryForward, ow, prefs.weightUnit);
				for (d = 0; d < m->days; d++) {
					m->weightArray[d] = cvtWeight(m->weightArray[d], ow, prefs.weightUnit);
				}
				hdPutMonthUnpacked(m, i);
			}
			hdDisposeMonthUnpacked(m);
		}

		//	Allocation may have moved form and list--recover pointers
		f = FrmGetActiveForm();
		lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthsMonthList));
	}
#endif


#ifdef FORCE_FEED

	/*	This code is a crude hack for verifying that the INT_TRENDFIT
		code in Trendfit.c doesn't overflow when confronted with a
		database in pounds with very large weights.  It takes a
		database in kilograms, multiplies all the values by 6, and
		changes the units to pounds.  */
		
	{
		int i, j;
		int Warning_FORCE_FEED_enabled;
		
		prefs.weightUnit = prefs.displayUnit = wPound;
		for (i = 0; i < n; i++) {
			struct hdMonth *m = hdGetMonthUnpacked(i);
			
			m->trendCarryForward *= 6;
			m->weightUnit = wPound;
			for (j = 0; j < m->days; j++) {
				m->weightArray[j] *= 6;
			}
			hdPutMonthUnpacked(m, i);
			hdDisposeMonthUnpacked(m);
		}

		//	Allocation may have moved form and list--recover pointers
		f = FrmGetActiveForm();
		lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthsMonthList));
	}
#endif

#ifdef SCRAMBLE_DB

	/*	The following code walks through the database and randomly
		changes the log units of months, converting the weight and
		trend carry-forward as necessary.  The trend carry-forward
		is changed by a random amount up to 10% above or below the
		correct value.  This code permits testing trend computation in
		the presence of heterogeneous units in the database, and the
		recomputation of trend from from weight entries where required.
		
		The first time the scrambler runs, its action is deterministic.
		Subsequent runs re-seed the pseudorandom number generator from
		TimGetTicks(), so each test with multiple scrambles will produce
		different results.  */
	
	{
		weightunit newUnit;
		int i, j;
		char ounit[16], nunit[16], sbuf[40];
		int Warning_Database_scrambler_enabled;
				
		for (i = 0; i < n; i++) {
			struct hdMonth *m = hdGetMonthUnpacked(i);
			
			stringListItem(WeightUnitAbbreviationsStringList, m->weightUnit, ounit, sizeof ounit);
			newUnit = (weightunit) (((rand() >> 7) & 0x7FFF) % 3);
			if (newUnit != m->weightUnit) {
				stringListItem(WeightUnitAbbreviationsStringList, newUnit, nunit, sizeof nunit);
				m->trendCarryForward = convertTrend(m->trendCarryForward,
													(weightunit) m->weightUnit, newUnit);
				if (m->trendCarryForward != 0) {
					m->trendCarryForward = (trend) (((double) m->trendCarryForward) +
													((((rand() >> 4) & 255) - 127.0) *
														((m->trendCarryForward * 0.10) / 127.0)
													)
												   );
														
				}
				for (j = 0; j < m->days; j++) {
					if (m->weightArray[j] != 0) {
						m->weightArray[j] = convertWeight(m->weightArray[j],
														  (weightunit) m->weightUnit, newUnit);
					}
				}
				m->weightUnit = newUnit;
				StrPrintF(sbuf, "%s -> %s", ounit, nunit);
			} else {
				StrPrintF(sbuf, "%s = %s", ounit, ounit);
			}
			m = hdAddComment(m, 0, sbuf);
			hdPutMonthUnpacked(m, i);
			hdDisposeMonthUnpacked(m);
		}

		//	Allocation may have moved form and list--recover pointers
		f = FrmGetActiveForm();
		lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, MonthsMonthList));
		FrmSetTitle(f, "DATABASE SCRAMBLER ON!");
		srand(TimGetTicks());
	}
#endif
	
	/*	Select the current month and scroll so it's visible.
		The Palm OS SDK reference says LstSetSelection is all
		you need, but don't you believe it.  */
	
	LstSetSelection(lp, thisMonth);
	LstMakeItemVisible(lp, thisMonth);
}